package sk.blade.globalvibratetoggle;

import android.app.AlertDialog;
import android.app.SearchManager;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.SharedPreferences;
import android.net.Uri;
import android.os.Build;
import android.os.Vibrator;
import android.preference.Preference;
import android.preference.PreferenceCategory;
import android.preference.PreferenceFragment;
import android.preference.PreferenceManager;
import android.preference.PreferenceScreen;
import android.preference.SwitchPreference;
import android.support.v7.app.ActionBarActivity;
import android.os.Bundle;
import android.text.Spannable;
import android.text.SpannableString;
import android.text.TextUtils;
import android.text.style.ForegroundColorSpan;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.widget.Toast;

import java.io.IOException;
import java.util.Arrays;
import java.util.HashSet;
import java.util.Set;
import java.util.zip.CRC32;


public class MainActivity extends ActionBarActivity {

    private PreferenceFragment prefsFrag;
    SharedPreferences prefs;
    SharedPreferences.OnSharedPreferenceChangeListener prefsListener;
    boolean demoGUIActive = false;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        setContentView(R.layout.activity_main);

        prefs = PreferenceManager.getDefaultSharedPreferences(getApplicationContext());
        prefsListener = new SharedPreferences.OnSharedPreferenceChangeListener() {
            @Override
            public void onSharedPreferenceChanged(SharedPreferences sharedPreferences, String key) {
                Log.i("main", "prefsChanged: "+key);
                if (sharedPreferences.getBoolean("auto_wifi_enabled", false) && (key.equals("auto_wifi_enabled") || key.equals("selectedWifis"))) {
                    // update vibrations according to current networks
                    Intent intent = new Intent(getApplicationContext(), WiFiStatusListener.class);
                    intent.setAction("sk.blade.globalvibratetoggle.UPDATE_ON_CHANGE");
                    sendBroadcast(intent);

                    new android.os.Handler().postDelayed(
                        new Runnable() {
                            public void run() {
                                SwitchPreference vibrateSwitch = (SwitchPreference) prefsFrag.findPreference("vibrate_enabled");
                                vibrateSwitch.setChecked(VibeManager.vibrationsEnabled());
                                updateWidgets();
                            }
                        }, 1000
                    );
                }

                if (key.equals("auto_wifi_enabled")) {
                    Preference wifisBtn = prefsFrag.findPreference("auto_wifi_networks");
                    if (wifisBtn != null) wifisBtn.setEnabled(sharedPreferences.getBoolean("auto_wifi_enabled", false));
                }

                if (key.equals("selectedWifis")) {
                    updateNetworksButton();
                }
            }
        };

        if (DemoTools.isDemoExpired(getApplicationContext())) {
            initDemoGUI();
        } else {
            initGUI();

            if (DemoTools.isDemo()) {
                DemoTools.scheduleNotification(getApplicationContext());
            }
        }

    }

    @Override
    public void onResume() {
        super.onResume();

        prefs.registerOnSharedPreferenceChangeListener(prefsListener);

        if (!prefs.getBoolean("suGranted", false)) {
            Thread thread = new Thread() {
                @Override
                public void run() {
                    try {
                        Process shell = Runtime.getRuntime().exec("su");
                        shell.waitFor();
                        if (shell.exitValue() == 0) {
                            SharedPreferences prefs = PreferenceManager.getDefaultSharedPreferences(getApplicationContext());
                            SharedPreferences.Editor editor = prefs.edit();
                            editor.putBoolean("suGranted", true);
                            editor.commit();
                        }
                    } catch (IOException e) {
                        e.printStackTrace();
                    } catch (InterruptedException e) {
                        e.printStackTrace();
                    }
                }
            };

            thread.start();
        }

        if (DemoTools.isDemoExpired(getApplicationContext())) {
            if (!demoGUIActive) initDemoGUI();
        } else {
            if (demoGUIActive) initGUI();
        }

        if (!demoGUIActive) {
            updateVibeToggleButton();
        }
    }

    @Override
    protected void onPause() {
        super.onPause();
        prefs.unregisterOnSharedPreferenceChangeListener(prefsListener);
    }

    private void initGUI() {
        prefsFrag = new PrefsFragment();
        getFragmentManager().beginTransaction().replace(android.R.id.content, prefsFrag).commit();
        getFragmentManager().executePendingTransactions();

        Preference wifisBtn = prefsFrag.findPreference("auto_wifi_networks");
        wifisBtn.setEnabled(prefs.getBoolean("auto_wifi_enabled", false));

        updateNetworksButton();

        bindButtonCallbacks();

        demoGUIActive = false;
    }

    private void initDemoGUI() {
        prefsFrag = new DemoPrefsFragment();
        getFragmentManager().beginTransaction().replace(android.R.id.content, prefsFrag).commit();
        getFragmentManager().executePendingTransactions();

        Preference expiredBtn = prefsFrag.findPreference("demo_expired");
        expiredBtn.setEnabled(false);

        Preference buyBtn = prefsFrag.findPreference("buy");
        buyBtn.setOnPreferenceClickListener(new Preference.OnPreferenceClickListener() {
            @Override
            public boolean onPreferenceClick(Preference preference) {
                final String appPackageName = "sk.blade.globalvibratetoggle";
                try {
                    startActivity(new Intent(Intent.ACTION_VIEW, Uri.parse("market://details?id=" + appPackageName)));
                } catch (android.content.ActivityNotFoundException anfe) {
                    startActivity(new Intent(Intent.ACTION_VIEW, Uri.parse("https://play.google.com/store/apps/details?id=" + appPackageName)));
                }

                return true;
            }
        });

        final Context context = getApplicationContext();

        Preference renewBtn = prefsFrag.findPreference("renew");
        renewBtn.setOnPreferenceClickListener(new Preference.OnPreferenceClickListener() {
            @Override
            public boolean onPreferenceClick(Preference preference) {
                DemoTools.renewTimestamp(context);
                DemoTools.scheduleNotification(context);
                Toast.makeText(context, "The demo period was renewed for 48 hours.", Toast.LENGTH_SHORT).show();

                boolean lastState = prefs.getBoolean("lastState", true);
                if (!lastState) {
                    VibeManager.setEnabled(false);
                    updateWidgets();
                }

                initGUI();

                return true;
            }
        });

        demoGUIActive = true;
    }

    private void updateNetworksButton() {
        Preference networksBtn = (Preference) prefsFrag.findPreference("auto_wifi_networks");
        if (networksBtn == null) return;

        Set<String> wifiPrefs = prefs.getStringSet("selectedWifis", new HashSet<String>(Arrays.asList("__all__")));
        boolean wifiPrefsAll = wifiPrefs.contains("__all__") ? true : false;
        boolean wifiPrefsNone = wifiPrefs.isEmpty();

        if (wifiPrefsAll) {
            networksBtn.setSummary("Disable vibrations on: ANY WIFI NETWORK");
        } else if (wifiPrefsNone) {
            networksBtn.setSummary("Disable vibrations on: (no networks selected)");
        } else {
            String wifis = TextUtils.join(", ", wifiPrefs);
            if (wifis.length() > 200) {
                wifis = wifis.substring(0, 196) + " ...";
            }

            networksBtn.setSummary("Disable vibrations on: " + wifis);
        }
    }

    private void updateVibeToggleButton() {
        SwitchPreference vibrateSwitch = (SwitchPreference) prefsFrag.findPreference("vibrate_enabled");
        vibrateSwitch.setChecked(VibeManager.vibrationsEnabled());
    }

    private void bindButtonCallbacks() {
        // root warning
        if (VibeManager.isRooted()) {
            PreferenceScreen parent = (PreferenceScreen) prefsFrag.findPreference("prefs");
            Preference rootRequired = prefsFrag.findPreference("root_required");
            parent.removePreference(rootRequired);
        } else {
            Preference rootRequired = prefsFrag.findPreference("root_required");

            Spannable title = new SpannableString(rootRequired.getTitle());
            title.setSpan( new ForegroundColorSpan(0xffe20000), 0, title.length(), 0 );
            rootRequired.setTitle(title);

            Spannable summary = new SpannableString(rootRequired.getSummary());
            summary.setSpan( new ForegroundColorSpan(0xffff8686), 0, summary.length(), 0 );
            rootRequired.setSummary(summary);

            rootRequired.setOnPreferenceClickListener(new Preference.OnPreferenceClickListener() {
                @Override
                public boolean onPreferenceClick(Preference preference) {
                    Intent intent = new Intent(Intent.ACTION_WEB_SEARCH);
                    intent.putExtra(SearchManager.QUERY, "how to root "+Build.MODEL); // query contains search string
                    startActivity(intent);

                    return true;
                }
            });
        }

        // toggle vibrate globally
        SwitchPreference vibrateSwitch = (SwitchPreference) prefsFrag.findPreference("vibrate_enabled");

        vibrateSwitch.setOnPreferenceChangeListener(new Preference.OnPreferenceChangeListener() {

            public boolean onPreferenceChange(Preference preference, Object newValue) {
                boolean checked = (Boolean) newValue;

                if (!VibeManager.setEnabled(checked)) {
                    showNotRootedAlert();
                } else {
                    SharedPreferences.Editor editor = prefs.edit();
                    editor.putBoolean("lastState", checked);
                    editor.apply();

                    updateWidgets();
                }
                return true;
            }
        });

        // test vibes

        Preference vibeButton = (Preference)prefsFrag.findPreference("vibrate_test");
        vibeButton.setOnPreferenceClickListener(new Preference.OnPreferenceClickListener() {
            @Override
            public boolean onPreferenceClick(Preference preference) {
                Vibrator v = (Vibrator) getSystemService(Context.VIBRATOR_SERVICE);
                v.vibrate(200);
                return true;
            }
        });

        // support
        Preference supportBtn = (Preference) prefsFrag.findPreference("support");
        supportBtn.setOnPreferenceClickListener(new Preference.OnPreferenceClickListener() {
            @Override
            public boolean onPreferenceClick(Preference preference) {
                String info = "Device: "+Build.BRAND+" "+Build.MODEL+" ("+Build.BOARD+"), " +
                        "OS: "+ Build.VERSION.RELEASE+", " +
                        "Rooted: "+(VibeManager.isRooted() ? "1" : "0");

                CRC32 crc = new CRC32();
                crc.update(info.getBytes());

                Intent emailIntent = new Intent(Intent.ACTION_SENDTO, Uri.fromParts("mailto", "android@blade.sk", null));
                emailIntent.putExtra(Intent.EXTRA_SUBJECT, "Global Vibrate Toggle Support");
                emailIntent.putExtra(Intent.EXTRA_TEXT, "\n\n-----------\n" + info + ", "+( (int)( crc.getValue() ) ));
                startActivity(Intent.createChooser(emailIntent, "Send email..."));

                return true;
            }
        });

        Preference rateBtn = (Preference) prefsFrag.findPreference("rate");
        rateBtn.setOnPreferenceClickListener(new Preference.OnPreferenceClickListener() {
            @Override
            public boolean onPreferenceClick(Preference preference) {
                final String appPackageName = getPackageName(); // getPackageName() from Context or Activity object
                try {
                    startActivity(new Intent(Intent.ACTION_VIEW, Uri.parse("market://details?id=" + appPackageName)));
                } catch (android.content.ActivityNotFoundException anfe) {
                    startActivity(new Intent(Intent.ACTION_VIEW, Uri.parse("https://play.google.com/store/apps/details?id=" + appPackageName)));
                }

                return true;
            }
        });

        if (DemoTools.isDemo()) {
            Preference fullVersionBtn = (Preference) prefsFrag.findPreference("full");
            fullVersionBtn.setOnPreferenceClickListener(new Preference.OnPreferenceClickListener() {
                @Override
                public boolean onPreferenceClick(Preference preference) {
                    final String fullVersionPkgName = "sk.blade.globalvibratetoggle";
                    try {
                        startActivity(new Intent(Intent.ACTION_VIEW, Uri.parse("market://details?id=" + fullVersionPkgName)));
                    } catch (android.content.ActivityNotFoundException anfe) {
                        startActivity(new Intent(Intent.ACTION_VIEW, Uri.parse("https://play.google.com/store/apps/details?id=" + fullVersionPkgName)));
                    }

                    return true;
                }
            });
        } else {
            PreferenceCategory parent = (PreferenceCategory) prefsFrag.findPreference("about");
            Preference fullVersionBtn = prefsFrag.findPreference("full");
            parent.removePreference(fullVersionBtn);
        }
    }


    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        // Inflate the menu; this adds items to the action bar if it is present.
        //getMenuInflater().inflate(R.menu.menu_main, menu);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        // Handle action bar item clicks here. The action bar will
        // automatically handle clicks on the Home/Up button, so long
        // as you specify a parent activity in AndroidManifest.xml.
        int id = item.getItemId();

        //noinspection SimplifiableIfStatement
        if (id == R.id.action_settings) {
            return true;
        }

        return super.onOptionsItemSelected(item);
    }

    private void showNotRootedAlert() {
        final Context thisContext = this;

        runOnUiThread(new Runnable() {
            @Override
            public void run() {
                if (!isFinishing()) {
                    new AlertDialog.Builder(thisContext)
                        .setTitle("Root required")
                        .setMessage("Can't change vibration settings. Your device doesn't seem to be rooted (SuperSU not found).")
                        .setPositiveButton("ok", new DialogInterface.OnClickListener() {
                            @Override
                            public void onClick(DialogInterface dialog, int which) {
                                // whatever...
                            }
                        }).create().show();
                }
            }
        });
    }

    private void updateWidgets() {
        Intent intent = new Intent(getApplicationContext(), VibeToggleWidget.class);
        intent.setAction(VibeToggleWidget.ACTION_UPDATE);
        getApplicationContext().sendBroadcast(intent);
    }

    public static class PrefsFragment extends PreferenceFragment {
        @Override
        public void onCreate(Bundle savedInstanceState) {
            super.onCreate(savedInstanceState);

            // Load the preferences from an XML resource
            addPreferencesFromResource(R.xml.prefs_main);
        }
    }

    public static class DemoPrefsFragment extends PreferenceFragment {
        @Override
        public void onCreate(Bundle savedInstanceState) {
            super.onCreate(savedInstanceState);

            // Load the preferences from an XML resource
            addPreferencesFromResource(R.xml.prefs_demo);
        }
    }
}
