package sk.blade.globalvibratetoggle;

import android.app.PendingIntent;
import android.appwidget.AppWidgetManager;
import android.appwidget.AppWidgetProvider;
import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.preference.PreferenceManager;
import android.util.Log;
import android.widget.RemoteViews;
import android.widget.Toast;


/**
 * Implementation of App Widget functionality.
 */
public class VibeToggleWidget extends AppWidgetProvider {

    public final static String ACTION_TOGGLE = "sk.blade.globalvibratetoggle.WIDGET_TOGGLE";
    public final static String ACTION_UPDATE = "sk.blade.globalvibratetoggle.WIDGET_UPDATE";
    protected boolean vibesEnabled;
    protected boolean needsUpdate = true; // android tuto triedu vytvara uplne lubovolne, cize toto je kvoli tomu, aby sme vedeli, kedy sa vibesEnabled persistol

    @Override
    public void onUpdate(Context context, AppWidgetManager appWidgetManager, int[] appWidgetIds) {
        Log.i("widget", "onUpdate");

        if (needsUpdate) {
            vibesEnabled = VibeManager.vibrationsEnabled();
        }

        // There may be multiple widgets active, so update all of them
        final int N = appWidgetIds.length;
        for (int i = 0; i < N; i++) {
            updateAppWidget(context, appWidgetManager, appWidgetIds[i]);
        }
    }


    @Override
    public void onEnabled(Context context) {
        // Enter relevant functionality for when the first widget is created
    }

    @Override
    public void onDisabled(Context context) {
        // Enter relevant functionality for when the last widget is disabled
    }

    protected void updateAppWidget(Context context, AppWidgetManager appWidgetManager, int appWidgetId) {
        // Construct the RemoteViews object
        RemoteViews views = new RemoteViews(context.getPackageName(), R.layout.vibe_toggle_widget);
        Intent intent = new Intent(context, getClass());
        intent.setAction(VibeToggleWidget.ACTION_TOGGLE);
        views.setOnClickPendingIntent(R.id.widgetLayout, PendingIntent.getBroadcast(context, 0, intent, 0));
        //views.setTextViewText(R.id.appwidget_text, widgetText);

       // views.setImageViewResource(R.id.imageView, R.drawable.ic_vibrate);

        if (vibesEnabled) {
            views.setImageViewResource(R.id.imageView, R.drawable.ic_vibrate_70);
        } else {
            views.setImageViewResource(R.id.imageView, R.drawable.ic_calm_70);
        }

        // Instruct the widget manager to update the widget
        appWidgetManager.updateAppWidget(appWidgetId, views);
    }

    @Override
    public void onReceive(Context context, Intent intent) {
        super.onReceive(context, intent);

        if (intent.getAction().equals(ACTION_TOGGLE)) {
            if (DemoTools.isDemoExpired(context)) {
                Toast.makeText(context, "The demo has expired. Open the app to extend it.", Toast.LENGTH_SHORT).show();
            } else {
                try {
                    vibesEnabled = VibeManager.toggleVibrations();
                    needsUpdate = false;

                    SharedPreferences.Editor prefsEditor = PreferenceManager.getDefaultSharedPreferences(context.getApplicationContext()).edit();
                    prefsEditor.putBoolean("lastState", vibesEnabled);
                    prefsEditor.apply();
                } catch (Exception e) {
                    Log.w("widget", e.getMessage());
                    Toast.makeText(context, "Failed to toggle vibrations. Not rooted?", Toast.LENGTH_SHORT).show();
                }
            }
        }

        if (intent.getAction().equals(ACTION_TOGGLE) || intent.getAction().equals(ACTION_UPDATE)) {
            // update widget
            AppWidgetManager appWidgetManager = AppWidgetManager.getInstance(context.getApplicationContext());
            ComponentName widget = new ComponentName(context.getApplicationContext(), getClass());
            int[] appWidgetIds = appWidgetManager.getAppWidgetIds(widget);
            if (appWidgetIds != null && appWidgetIds.length > 0) {
                onUpdate(context, appWidgetManager, appWidgetIds);
            }
        }
    }
}


