package sk.blade.globalvibratetoggle;

import android.app.AlarmManager;
import android.app.Notification;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.net.ConnectivityManager;
import android.net.wifi.WifiInfo;
import android.net.wifi.WifiManager;
import android.os.Build;
import android.preference.PreferenceManager;
import android.provider.Settings;
import android.util.Log;

import java.util.Arrays;
import java.util.HashSet;
import java.util.Set;

/**
 * Created by 81403 on 27.6.2015.
 */
public class WiFiStatusListener extends BroadcastReceiver {
    private String tag = "wlistener";

    @Override
    public void onReceive(Context context, Intent intent) {
        if (intent.getAction().equals("android.net.conn.CONNECTIVITY_CHANGE")) {
            if (DemoTools.isDemoExpired(context)) return;

            // this intent is fired multiple times in succession, we need to throttle it by using an alarm
            PendingIntent updateStatePendingIntent = PendingIntent.getBroadcast(context, 0, new Intent("sk.blade.globalvibratetoggle.UPDATE_ON_CHANGE"), 0);

            AlarmManager alarmManager = (AlarmManager) context.getSystemService(Context.ALARM_SERVICE);
            if(Build.VERSION.SDK_INT >= Build.VERSION_CODES.KITKAT) {
                alarmManager.setExact(AlarmManager.RTC_WAKEUP, System.currentTimeMillis() + 2000, updateStatePendingIntent);
            } else {
                alarmManager.set(AlarmManager.RTC_WAKEUP, System.currentTimeMillis() + 2000, updateStatePendingIntent);
            }
        } else if (intent.getAction().equals("sk.blade.globalvibratetoggle.UPDATE_ON_CHANGE")) {
            Log.i(tag, "update on change");
            detectStateAndApply(context);
        } else if (intent.getAction().equals("sk.blade.globalvibratetoggle.NOTIFY_EXPIRED")) {
            Log.i(tag, "trial expired, posting notification");
            postTrialExpiredNotification(context);
        }
    }

    private void detectStateAndApply(Context context) {
        SharedPreferences prefs = PreferenceManager.getDefaultSharedPreferences(context.getApplicationContext());
        if (!prefs.getBoolean("auto_wifi_enabled", false)) {
            return;
        }

        SharedPreferences.Editor prefsEditor = prefs.edit();
        boolean airplaneMode = false;

        try {
            if (Settings.System.getInt(context.getContentResolver(), Settings.Global.AIRPLANE_MODE_ON) != 0) {
                airplaneMode = true;
            }
        } catch (Settings.SettingNotFoundException e) {
            try {
                // legacy name
                if (Settings.System.getInt(context.getContentResolver(), Settings.System.AIRPLANE_MODE_ON) != 0) {
                    airplaneMode = true;
                }
            } catch (Settings.SettingNotFoundException e2) {
            }
        }

        final ConnectivityManager connMgr = (ConnectivityManager) context.getSystemService(Context.CONNECTIVITY_SERVICE);
        final android.net.NetworkInfo wifi = connMgr.getNetworkInfo(ConnectivityManager.TYPE_WIFI);

        Log.i(tag, "WiFi: " + (wifi.isConnected() ? "connected" : "disconnected") + "; Airplane mode: "+(airplaneMode ? "on" : "off"));

        boolean isKnownWifi = false;

        if (wifi.isConnected()) { // check if we're on a known wifi (or if all the wifis should be treated as known)
            final WifiManager wifiManager = (WifiManager) context.getSystemService(Context.WIFI_SERVICE);
            final WifiInfo connectionInfo = wifiManager.getConnectionInfo();
            String ssid = connectionInfo.getSSID();
            if (ssid.charAt(0) == '"') { // named wifis are wrapped in double quotes
                ssid = ssid.substring(1, ssid.length() - 1);
            }

            Set<String> wifiPrefs = prefs.getStringSet("selectedWifis", new HashSet<String>(Arrays.asList("__all__")));
            if (wifiPrefs != null) {
                isKnownWifi = wifiPrefs.contains("__all__")  || wifiPrefs.contains(ssid);
            }

            Log.i(tag, "Known wifi: " + (isKnownWifi ? "y" : "n") + "; SSID: " + ssid);
        }

        if ((wifi.isConnected() && isKnownWifi) || airplaneMode) {
            Log.i(tag, "Vibes OFF");
            if (VibeManager.setEnabled(false)) {
                prefsEditor.putBoolean("lastState", false);
                prefsEditor.apply();
            }
        } else {
            Log.i(tag, "Vibes ON");
            if (VibeManager.setEnabled(true)) {
                prefsEditor.putBoolean("lastState", true);
                prefsEditor.apply();
            }
        }

        Intent intent = new Intent(context.getApplicationContext(), VibeToggleWidget.class);
        intent.setAction(VibeToggleWidget.ACTION_UPDATE);
        context.sendBroadcast(intent);
    }

    private void postTrialExpiredNotification(Context context) {
        Intent notificationIntent = new Intent(context, MainActivity.class);
        PendingIntent contentIntent = PendingIntent.getActivity(context, 0, notificationIntent, PendingIntent.FLAG_CANCEL_CURRENT);

        NotificationManager nm = (NotificationManager) context.getSystemService(Context.NOTIFICATION_SERVICE);

        Notification.Builder builder = new Notification.Builder(context);

        builder.setContentIntent(contentIntent)
                .setSmallIcon(R.drawable.ic_calm_notif_icon)
                //.setLargeIcon((BitmapFactory.decodeResource(res, R.drawable.some_big_img)))
                .setTicker("Global Vibration Toggle demo has expired. Tap for more info.")
                .setWhen(System.currentTimeMillis())
                .setAutoCancel(true)
                .setContentTitle("Demo expired")
                .setContentText("Tap for more info.")
                .setStyle(new Notification.BigTextStyle().bigText(
                    "Global Vibration Toggle demo has expired. Tap for more info."
                ));
        Notification n = builder.build();

        nm.notify(0, n);

        // reset vibration to default
        VibeManager.setEnabled(true);

        Intent intent = new Intent(context.getApplicationContext(), VibeToggleWidget.class);
        intent.setAction(VibeToggleWidget.ACTION_UPDATE);
        context.sendBroadcast(intent);
    }
}