package sk.blade.globalvibratetoggle;

import android.content.Context;
import android.content.SharedPreferences;
import android.net.wifi.WifiConfiguration;
import android.net.wifi.WifiManager;
import android.preference.PreferenceManager;
import android.support.v7.app.ActionBarActivity;
import android.os.Bundle;
import android.util.TypedValue;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.Button;
import android.widget.CheckBox;
import android.widget.LinearLayout;
import android.widget.TextView;

import java.util.Arrays;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashSet;
import java.util.List;
import java.util.Set;


public class WifiSelectActivity extends ActionBarActivity {

    private boolean saveOnExit = true;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_wifi_select);

        SharedPreferences prefs = PreferenceManager.getDefaultSharedPreferences(getApplicationContext());
        Set<String> wifiPrefs = prefs.getStringSet("selectedWifis", new HashSet<String>(Arrays.asList("__all__")));
        boolean wifiPrefsAll = wifiPrefs.contains("__all__") ? true : false;

        LinearLayout container = (LinearLayout) findViewById(R.id.checkboxContainer);
        WifiManager wm = (WifiManager) getSystemService(Context.WIFI_SERVICE);

        List<WifiConfiguration> wifis = wm.getConfiguredNetworks();
        if (wifis == null) {
            saveOnExit = false;

            TextView error = new TextView(this);
            error.setText("Error - could not retrieve saved WiFi networks. Make sure WiFi is enabled.");
            error.setTextColor(0xffff0000);
            //error.setLayoutParams(new LinearLayout.LayoutParams(LayoutParams.FILL_PARENT, LayoutParams.WRAP_CONTENT));
            container.addView(error);

            return;
        }

        Collections.sort(wifis, new Comparator<WifiConfiguration>() {
            @Override
            public int compare(WifiConfiguration o1, WifiConfiguration o2) {
                return o1.SSID.toLowerCase().compareTo(o2.SSID.toLowerCase());
            }
        });

        final float scale = getResources().getDisplayMetrics().density;

        for (WifiConfiguration wifi : wifis) {
            String ssid;
            if (wifi.SSID.charAt(0) == '"') { // named wifis are wrapped in double quotes
                ssid = wifi.SSID.substring(1, wifi.SSID.length() - 1);
            } else {
                ssid = wifi.SSID;
            }

            CheckBox cb = new CheckBox(this);
            cb.setText(ssid);
            cb.setTextSize(TypedValue.COMPLEX_UNIT_DIP, 16.0f);
            cb.setPadding(0, (int)(10 * scale), 0, (int)(10 * scale));
            cb.setChecked(wifiPrefsAll || wifiPrefs.contains(ssid));
            container.addView(cb);
        }

        initButtons();
    }

    @Override
    protected void onDestroy() {
        if (saveOnExit) {
            int checkboxes = 0, checked = 0;
            HashSet<String> wifiPrefs = new HashSet<String>();

            LinearLayout container = (LinearLayout) findViewById(R.id.checkboxContainer);
            for (int i = 0; i < container.getChildCount(); i++) {
                CheckBox cb = (CheckBox) container.getChildAt(i);
                checkboxes++;
                if (cb.isChecked()) {
                    checked++;
                    wifiPrefs.add(cb.getText().toString());
                }
            }

            SharedPreferences prefs = PreferenceManager.getDefaultSharedPreferences(getApplicationContext());
            SharedPreferences.Editor editor = prefs.edit();

            if (checked == checkboxes) {
                editor.putStringSet("selectedWifis", new HashSet<String>(Arrays.asList("__all__")));
            } else {
                editor.putStringSet("selectedWifis", wifiPrefs);
            }

            editor.commit();
        }

        super.onDestroy();
    }

    protected void initButtons() {
        Button selectAll = (Button) findViewById(R.id.btn_select_all);
        selectAll.setOnClickListener(new View.OnClickListener() {
            public void onClick(View v) {
                LinearLayout container = (LinearLayout) findViewById(R.id.checkboxContainer);
                for (int i = 0; i < container.getChildCount(); i++) {
                    CheckBox cb = (CheckBox) container.getChildAt(i);
                    cb.setChecked(true);
                }
            }
        });

        Button selectNone = (Button) findViewById(R.id.btn_select_none);
        selectNone.setOnClickListener(new View.OnClickListener() {
            public void onClick(View v) {
                LinearLayout container = (LinearLayout) findViewById(R.id.checkboxContainer);
                for (int i = 0; i < container.getChildCount(); i++) {
                    CheckBox cb = (CheckBox) container.getChildAt(i);
                    cb.setChecked(false);
                }
            }
        });
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        // Inflate the menu; this adds items to the action bar if it is present.
        //getMenuInflater().inflate(R.menu.menu_wifi_select, menu);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        // Handle action bar item clicks here. The action bar will
        // automatically handle clicks on the Home/Up button, so long
        // as you specify a parent activity in AndroidManifest.xml.
        int id = item.getItemId();

        //noinspection SimplifiableIfStatement
        if (id == R.id.action_settings) {
            return true;
        }

        return super.onOptionsItemSelected(item);
    }
}
